﻿using System;
using System.ComponentModel;
using System.Drawing;
using System.IO;
using System.Windows.Forms;
using MarkusGrothoff.Windows.Forms;

namespace MarkusGrothoff.Media.Controls {

    /// <summary>
    /// Stellen Controls zur Steuerung von Soundplayern dar.
    /// </summary>
    public partial class SoundPlayerUserControl : UserControl {

        /// <summary>
        /// Datei wurde abgespielt.
        /// </summary>
        public event EventHandler Played = null;

        private readonly Image volumeDisabledImage = Properties.Resources.VolumeDisabled;
        private readonly Image volumeNormalImage = Properties.Resources.VolumeNormal;
        private readonly Image volumeHoverImage = Properties.Resources.VolumeHot;
        private readonly Image muteDisabledImage = Properties.Resources.MuteDisabled;
        private readonly Image muteNormalImage = Properties.Resources.MuteNormal;
        private readonly Image muteHoverImage = Properties.Resources.MuteHot;
        private readonly Image pauseDisabledImage = Properties.Resources.PauseDisabled;
        private readonly Image pauseNormalImage = Properties.Resources.PauseNormal;
        private readonly Image pauseHoverImage = Properties.Resources.PauseHot;
        private readonly Image playDisabledImage = Properties.Resources.PlayDisabled;
        private readonly Image playNormalImage = Properties.Resources.PlayNormal;
        private readonly Image playHoverImage = Properties.Resources.PlayHot;

        private readonly MediaPlayer player = new MediaPlayer();

        /// <summary>
        /// Erstellt einen Soundplayer.
        /// </summary>
        public SoundPlayerUserControl() {
            InitializeComponent();
            player.Played += new EventHandler(player_Played);
            Disposed += new EventHandler(SoundPlayerUserControl_Disposed);
            FileNameLabel.Text = String.Empty;
            PositionLengthLabel.Text = String.Empty;
            UpdateControls();
        }

        /// <summary>
        /// Ruft die geöffnete Datei ab oder legt sie fest.
        /// Ist keine Datei geöffnet, wird String.Empty zurückgegeben.
        /// Wird String.Empty übergeben, wird eine geöffnete Datei geschlossen.
        /// </summary>
        /// <exception cref="MediaException">Der Befehl konnte nicht ausgeführt werden.</exception>
        public String FileName {
            get {
                return player.FileName;
            }
            set {
                if (player.Status != MediaStatus.Closed) player.Close();
                if (value.Length > 0) {
                    if (!File.Exists(value)) {
                        MessageBox.Show(string.Format("Die Datei {0} konnte nicht gefunden werden.", value),
                            "Datei nicht gefunden", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    } else {
                        player.Open(value);
                        UpdateControls();
                    }
                }
                if (AutoSize) PerformLayout();
            }
        }

        /// <summary>
        /// Ruft den Status des Soundplayers ab.
        /// </summary>
        public MediaStatus Status {
            get {
                try {
                    return player.Status;
                } catch (MediaException ex) {
                    ShowMediaException(ex);
                    return MediaStatus.Closed;
                }
            }
        }

        /// <summary>
        /// Startet die Wiedergabe an der aktuellen Position.
        /// </summary>
        /// <exception cref="InvalidOperationException">Keine Datei geöffnet.</exception>
        public void Play() {
            try {
                player.Play(player.Position);
                UpdateControls();
            } catch (MediaException ex) {
                ShowMediaException(ex);
            }
        }

        /// <summary>
        /// Stoppt die Wiedergabe und geht an den Anfang der Datei.
        /// </summary>
        /// <exception cref="InvalidOperationException">Keine Datei geöffnet.</exception>
        public void Stop() {
            try {
                player.Stop();
                player.Position = TimeSpan.Zero;
                UpdateControls();
            } catch (MediaException ex) {
                ShowMediaException(ex);
            }
        }

        /// <summary>
        /// Pausiert die Wiedergabe.
        /// </summary>
        /// <exception cref="InvalidOperationException">Keine Datei geöffnet.</exception>
        public void Pause() {
            try {
                player.Pause();
                UpdateControls();
            } catch (MediaException ex) {
                ShowMediaException(ex);
            }
        }

        /// <summary>
        /// Ruft ab, ob die Audiowiedergabe stummgeschaltet ist, oder legt es fest.
        /// </summary>
        /// <exception cref="InvalidOperationException">Keine Datei geöffnet.</exception>
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public bool IsMuted {
            get {
                try {
                    return player.IsMuted;
                } catch (MediaException ex) {
                    ShowMediaException(ex);
                    return true;
                }
            }
            set {
                try {
                    player.IsMuted = value;
                    UpdateControls();
                } catch (MediaException ex) {
                    ShowMediaException(ex);
                }
            }
        }

        /// <summary>
        /// Ruft die Lautstärke beider Kanäle ab oder legt sie fest.
        /// </summary>
        /// <exception cref="InvalidOperationException">Keine Datei geöffnet.</exception>
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public double Volume {
            get {
                try {
                    return player.Volume;
                } catch (MediaException ex) {
                    ShowMediaException(ex);
                    return 0.0;
                }
            }
            set {
                try {
                    double volume = value < 0.0 ? 0.0 : value > 1.0 ? 1.0 : value;
                    player.Volume = volume;
                    UpdateControls();
                } catch (MediaException ex) {
                    ShowMediaException(ex);
                }
            }
        }

        /// <summary>
        /// Ruft die Position in der Datei ab oder legt sie fest.
        /// </summary>
        /// <exception cref="InvalidOperationException">Keine Datei geöffnet.</exception>
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public TimeSpan Position {
            get {
                try {
                    return player.Position;
                } catch (MediaException ex) {
                    ShowMediaException(ex);
                    return TimeSpan.Zero;
                }
            }
            set {
                try {
                    TimeSpan position = value < TimeSpan.Zero ? TimeSpan.Zero : value > player.Length ? player.Length : value;
                    player.Position = position;
                    UpdateControls();
                } catch (MediaException ex) {
                    ShowMediaException(ex);
                }
            }
        }

        /// <summary>
        /// Ruft die Gesamtspiellänge der Datei ab.
        /// </summary>
        /// <exception cref="InvalidOperationException">Keine Datei geöffnet.</exception>
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public TimeSpan Length {
            get {
                try {
                    return player.Length;
                } catch (MediaException ex) {
                    ShowMediaException(ex);
                    return TimeSpan.Zero;
                }
            }
        }

        /// <summary>
        /// Löst das Played-Event aus.
        /// </summary>
        protected virtual void OnPlayed(EventArgs e) {
            if (Played != null) {
                EventHandler played = (EventHandler)Played.Clone();
                if (played != null) played(this, e);
            }
        }

        /// <summary>
        /// Zeigt eine Media-Exception als Meldung an.
        /// </summary>
        /// <param name="ex">Media-Exception.</param>        
        private void ShowMediaException(MediaException ex) {
            MessageBox.Show(String.Format("Folgender Fehler ist aufgetreten:{0}{1}", Environment.NewLine + Environment.NewLine, ex.Message),
                            "Sorry :-(", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }

        /// <summary>
        /// Aktualisiert die Controls entsprechend dem Status.
        /// </summary>
        private void UpdateControls() {
            try {
                MediaStatus status = player.Status;

                // Informationsbereich oben.
                if (status == MediaStatus.Closed) {
                    FileNameLabel.Text = String.Empty;
                    PositionLengthLabel.Text = String.Empty;
                } else {
                    FileNameLabel.Text = player.FileName;
                    TimeSpan position = player.Position;
                    TimeSpan length = player.Length;
                    PositionLengthLabel.Text = String.Format("{0:0}:{1:00} / {2:0}:{3:00}", position.Minutes, position.Seconds, length.Minutes, length.Seconds);
                    ProgressTrackBar.ValueChanged -= ProgressTrackBar_ValueChanged;
                    ProgressTrackBar.Maximum = (int)length.TotalMilliseconds;
                    ProgressTrackBar.Value = (int)position.TotalMilliseconds;
                    ProgressTrackBar.ValueChanged += new EventHandler(ProgressTrackBar_ValueChanged);
                }
                ProgressTrackBar.Enabled = status != MediaStatus.Closed;

                // Unterer Bereich (Buttons).
                PlayPauseButton.Enabled = status != MediaStatus.Closed;
                if (status == MediaStatus.Playing) {
                    PlayPauseButton.DisabledImage = pauseDisabledImage;
                    PlayPauseButton.NormalImage = pauseNormalImage;
                    PlayPauseButton.HoverImage = pauseHoverImage;
                } else {
                    PlayPauseButton.DisabledImage = playDisabledImage;
                    PlayPauseButton.NormalImage = playNormalImage;
                    PlayPauseButton.HoverImage = playHoverImage;
                }
                StopButton.Enabled = status != MediaStatus.Closed;
                MuteButton.Enabled = status != MediaStatus.Closed;

                if (status == MediaStatus.Closed || !player.IsMuted) {
                    MuteButton.DisabledImage = volumeDisabledImage;
                    MuteButton.NormalImage = volumeNormalImage;
                    MuteButton.HoverImage = volumeHoverImage;
                } else {
                    MuteButton.DisabledImage = muteDisabledImage;
                    MuteButton.NormalImage = muteNormalImage;
                    MuteButton.HoverImage = muteHoverImage;
                }
                VolumeTrackBar.Enabled = status != MediaStatus.Closed;

                VolumeTrackBar.ValueChanged -= VolumeTrackBar_ValueChanged;
                if (status != MediaStatus.Closed) {
                    VolumeTrackBar.Value = (int)(player.Volume * 1000.0);
                } else {
                    VolumeTrackBar.Value = 0;
                }
                VolumeTrackBar.ValueChanged += new EventHandler(VolumeTrackBar_ValueChanged);

                UpdateTimer.Enabled = status != MediaStatus.Closed;
            } catch (MediaException) { }
        }

        private void SoundPlayerUserControl_Disposed(object sender, EventArgs e) { player.Dispose(); }

        private void UpdateTimer_Tick(object sender, EventArgs e) { UpdateControls(); }

        private void MuteButton_Click(object sender, EventArgs e) {
            player.IsMuted = !player.IsMuted;
            if (player.IsMuted) {
                VolumeTrackBar.ValueChanged -= VolumeTrackBar_ValueChanged;
                VolumeTrackBar.Value = 0;
                player.Volume = 0.0;
                VolumeTrackBar.ValueChanged += new EventHandler(VolumeTrackBar_ValueChanged);
            }
            UpdateControls();
        }

        private void StopButton_Click(object sender, EventArgs e) { Stop(); }

        private void PlayPauseButton_Click(object sender, EventArgs e) {
            if (player.Status == MediaStatus.Playing) {
                Pause();
            } else {
                Play();
            }
        }

        private void VolumeTrackBar_ValueChanged(object sender, EventArgs e) {
            if (player.IsMuted) { player.IsMuted = false; }
            player.Volume = ((TrackBar)sender).Value * 0.001;
            UpdateControls();
        }

        private void ProgressTrackBar_ValueChanged(object sender, EventArgs e) { player.Position = TimeSpan.FromMilliseconds(((TrackBar)sender).Value); }

        private void player_Played(object sender, EventArgs e) {
            player.Position = TimeSpan.Zero;
            UpdateControls();
            OnPlayed(new EventArgs());
        }
    }
}