﻿using System;
using System.ComponentModel;
using System.Drawing;
using System.IO;
using System.Windows.Forms;
using MarkusGrothoff.Windows.Forms;

namespace MarkusGrothoff.Media.Controls {

    /// <summary>
    /// Stellen Videobereiche dar.
    /// </summary>
    public partial class VideoPaneUserControl : UserControl {

        /// <summary>
        /// Datei wurde abgespielt.
        /// </summary>
        public event EventHandler Played = null;

        private readonly VideoPlayer player;

        /// <summary>
        /// Erstellt einen Videobereich.
        /// </summary>
        public VideoPaneUserControl() {
            InitializeComponent();
            player.Played += new EventHandler(player_Played);
            Disposed += new EventHandler(VideoPaneUserControl_Disposed);
            player = new VideoPlayer(Handle);
        }

        /// <summary>
        /// Ruft die geöffnete Datei ab oder legt sie fest.
        /// Ist keine Datei geöffnet, wird String.Empty zurückgegeben.
        /// Wird String.Empty übergeben, wird eine geöffnete Datei geschlossen.
        /// </summary>    
        /// <exception cref="MediaException">Der Befehl konnte nicht ausgeführt werden.</exception>
        public String FileName {
            get {
                return player.FileName;
            }
            set {
                if (player.Status != MediaStatus.Closed) player.Close();
                if (value.Length > 0) {
                    if (!File.Exists(value)) {
                        MessageBox.Show(string.Format("Die Datei {0} konnte nicht gefunden werden.", value),
                            "Datei nicht gefunden", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    } else {
                        player.Open(value);
                    }
                }
                if (AutoSize) PerformLayout();
            }
        }

        /// <summary>
        /// Ruft den Status des Soundplayers ab.
        /// </summary>
        public MediaStatus Status {
            get {
                try {
                    return player.Status;
                } catch (MediaException ex) {
                    ShowMediaException(ex);
                    return MediaStatus.Closed;
                }
            }
        }

        /// <summary>
        /// Startet die Wiedergabe an der aktuellen Position.
        /// </summary>
        /// <exception cref="InvalidOperationException">Keine Datei geöffnet.</exception>
        public void Play() {
            try {
                player.Play(player.Position);
            } catch (MediaException ex) {
                ShowMediaException(ex);
            }
        }

        /// <summary>
        /// Stoppt die Wiedergabe und geht an den Anfang der Datei.
        /// </summary>
        /// <exception cref="InvalidOperationException">Keine Datei geöffnet.</exception>
        public void Stop() {
            try {
                player.Stop();
                player.Position = TimeSpan.Zero;
            } catch (MediaException ex) {
                ShowMediaException(ex);
            }
        }

        /// <summary>
        /// Pausiert die Wiedergabe.
        /// </summary>
        /// <exception cref="InvalidOperationException">Keine Datei geöffnet.</exception>
        public void Pause() {
            try {
                player.Pause();
            } catch (MediaException ex) {
                ShowMediaException(ex);
            }
        }

        /// <summary>
        /// Ruft ab, ob die Audiowiedergabe stummgeschaltet ist, oder legt es fest.
        /// </summary>
        /// <exception cref="InvalidOperationException">Keine Datei geöffnet.</exception>
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public bool IsMuted {
            get {
                try {
                    return player.IsMuted;
                } catch (MediaException ex) {
                    ShowMediaException(ex);
                    return true;
                }
            }
            set {
                try {
                    player.IsMuted = value;
                } catch (MediaException ex) {
                    ShowMediaException(ex);
                }
            }
        }

        /// <summary>
        /// Ruft die Lautstärke beider Kanäle ab oder legt sie fest.
        /// </summary>
        /// <exception cref="InvalidOperationException">Keine Datei geöffnet.</exception>
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public double Volume {
            get {
                try {
                    return player.Volume;
                } catch (MediaException ex) {
                    ShowMediaException(ex);
                    return 0.0;
                }
            }
            set {
                try {
                    double volume = value < 0.0 ? 0.0 : value > 1.0 ? 1.0 : value;
                    player.Volume = volume;
                } catch (MediaException ex) {
                    ShowMediaException(ex);
                }
            }
        }

        /// <summary>
        /// Ruft die Position in der Datei ab oder legt sie fest.
        /// </summary>
        /// <exception cref="InvalidOperationException">Keine Datei geöffnet.</exception>
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public TimeSpan Position {
            get {
                try {
                    return player.Position;
                } catch (MediaException ex) {
                    ShowMediaException(ex);
                    return TimeSpan.Zero;
                }
            }
            set {
                try {
                    TimeSpan position = value < TimeSpan.Zero ? TimeSpan.Zero : value > player.Length ? player.Length : value;
                    player.Position = position;
                } catch (MediaException ex) {
                    ShowMediaException(ex);
                }
            }
        }

        /// <summary>
        /// Ruft die Gesamtspiellänge der Datei ab.
        /// </summary>
        /// <exception cref="InvalidOperationException">Keine Datei geöffnet.</exception>
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public TimeSpan Length {
            get {
                try {
                    return player.Length;
                } catch (MediaException ex) {
                    ShowMediaException(ex);
                    return TimeSpan.Zero;
                }
            }
        }

        /// <summary>
        /// Ruft die bevorzugte Größe der Komponente ab.
        /// </summary>
        public override Size GetPreferredSize(Size proposedSize) {
            return player == null || player.Status == MediaStatus.Closed ? base.GetPreferredSize(proposedSize) : player.Size;
        }

        /// <summary>
        /// Löst das Played-Event aus.
        /// </summary>
        protected virtual void OnPlayed(EventArgs e) {
            if (Played != null) {
                EventHandler played = (EventHandler)Played.Clone();
                if (played != null) played(this, e);
            }
        }

        /// <summary>
        /// Zeigt eine Media-Exception als Meldung an.
        /// </summary>
        /// <param name="ex">Media-Exception.</param>        
        private void ShowMediaException(MediaException ex) {
            MessageBox.Show(String.Format("Folgender Fehler ist aufgetreten:{0}{1}", Environment.NewLine + Environment.NewLine, ex.Message),
                            "Sorry :-(", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }

        private void VideoPaneUserControl_Disposed(Object sender, EventArgs e) { player.Dispose(); }

        private void player_Played(object sender, EventArgs e) {
            player.Position = TimeSpan.Zero;
            OnPlayed(new EventArgs());
        }
    }
}